export MPIScanner, getDAQ, getGaussMeter, getRobot, getSafety, getGeneralParams,
getSurveillanceUnit, getTemperatureSensor

mutable struct MPIScanner
  file::String
  params::Dict
  generalParams::Dict
  daq::Union{AbstractDAQ,Nothing}
  robot::Union{Robot,Nothing}
  gaussmeter::Union{GaussMeter,Nothing}
  safty::Union{RobotSetup,Nothing}
  surveillanceUnit::Union{SurveillanceUnit,Nothing}
  temperatureSensor::Union{TemperatureSensor,Nothing}
  recoMethod::Function

  function MPIScanner(file::String)
    filename = joinpath(@__DIR__, "Configurations", file)
    params = TOML.parsefile(filename)
    generalParams = params["General"]
    return new(file,params,generalParams,nothing,nothing,nothing,nothing,nothing,nothing,()->())
  end
end

function Base.close(scanner::MPIScanner)
  if scanner.robot != nothing
    close(scanner.robot)
  end
  if scanner.gaussmeter != nothing
    close(scanner.gaussmeter)
  end
  if scanner.surveillanceUnit != nothing
    close(scanner.surveillanceUnit)
  end
  if scanner.temperatureSensor != nothing
    close(scanner.temperatureSensor)
  end

end

getGeneralParams(scanner::MPIScanner) = scanner.generalParams

function getDAQ(scanner::MPIScanner)
  if scanner.daq == nothing && haskey(scanner.params, "DAQ")
    scanner.daq = DAQ(scanner.params["DAQ"])
  end
  return scanner.daq
end

function getRobot(scanner::MPIScanner)
  if scanner.robot == nothing && haskey(scanner.params, "Robot")
    scanner.robot = Robot(scanner.params["Robot"])
  end
  return scanner.robot
end

function getGaussMeter(scanner::MPIScanner)
  if scanner.gaussmeter == nothing && haskey(scanner.params, "GaussMeter")
    scanner.gaussmeter = GaussMeter(scanner.params["GaussMeter"])
  end
  return scanner.gaussmeter
end

function getSafety(scanner::MPIScanner)
  if scanner.safty == nothing && haskey(scanner.params, "Safety")
    scanner.safty = RobotSetup(scanner.params["Safety"])
  end
  return scanner.safty
end

function getSurveillanceUnit(scanner::MPIScanner)
  if scanner.surveillanceUnit == nothing && haskey(scanner.params, "SurveillanceUnit")
    scanner.surveillanceUnit = SurveillanceUnit(scanner.params["SurveillanceUnit"])
  end
  return scanner.surveillanceUnit
end


function getTemperatureSensor(scanner::MPIScanner)
  if scanner.temperatureSensor == nothing && haskey(scanner.params, "TemperatureSensor")
    scanner.temperatureSensor = TemperatureSensor(scanner.params["TemperatureSensor"])
  end
  return scanner.temperatureSensor
end
